@@ -14,7 +14,7 @@ const Tweener = imports.ui.tweener;
 
 const POPUP_APPICON_SIZE = 96;
 const POPUP_SCROLL_TIME = 0.10; // seconds
-const POPUP_FADE_IN_TIME = 0.4; // seconds
+const POPUP_DELAY_TIMEOUT = 150; // milliseconds
 const POPUP_FADE_OUT_TIME = 0.1; // seconds
 
 const APP_ICON_HOVER_TIMEOUT = 200; // milliseconds
@@ -31,6 +31,18 @@ function mod(a, b) {
     return (a + b) % b;
 }
 
+function primaryModifier(mask) {
+    if (mask == 0)
+        return 0;
+
+    let primary = 1;
+    while (mask > 1) {
+        mask >>= 1;
+        primary <<= 1;
+    }
+    return primary;
+}
+
 function AltTabPopup() {
     this._init();
 }
@@ -48,11 +60,13 @@ AltTabPopup.prototype = {
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
 
         this._haveModal = false;
+        this._modifierMask = 0;
 
         this._currentApp = 0;
         this._currentWindow = -1;
         this._thumbnailTimeoutId = 0;
         this._motionTimeoutId = 0;
+        this._initialDelayTimeoutId = 0;
 
         this.thumbnailsVisible = false;
 
@@ -120,9 +134,9 @@ AltTabPopup.prototype = {
         }
     },
 
-    show : function(backward, binding) {
-        let tracker = Shell.WindowTracker.get_default();
-        let apps = tracker.get_running_apps ('');
+    show : function(backward, binding, mask) {
+        let appSys = Shell.AppSystem.get_default();
+        let apps = appSys.get_running ();
 
         if (!apps.length)
             return false;
@@ -130,6 +144,7 @@ AltTabPopup.prototype = {
         if (!Main.pushModal(this.actor))
             return false;
         this._haveModal = true;
+        this._modifierMask = primaryModifier(mask);
 
         this.actor.connect('key-press-event', Lang.bind(this, this._keyPressEvent));
         this.actor.connect('key-release-event', Lang.bind(this, this._keyReleaseEvent));
@@ -178,21 +193,18 @@ AltTabPopup.prototype = {
         // details.) So we check now. (Have to do this after updating
         // selection.)
         let [x, y, mods] = global.get_pointer();
-        if (!(mods & Gdk.ModifierType.MOD1_MASK)) {
+        if (!(mods & this._modifierMask)) {
             this._finish();
             return false;
         }
 
-        // Using easeInOutExpo over 400ms gives us 150ms of "nearly
-        // invisible" (less than 10% opacity), followed by a 100ms
-        // tween in (to 90% opacity, with the last 10% coming over the
-        // next 150ms). So if the user releases Alt quickly after we
-        // start tweening, they'll never see the switcher.
-        Tweener.addTween(this.actor,
-                         { opacity: 255,
-                           time: POPUP_FADE_IN_TIME,
-                           transition: 'easeInOutExpo'
-                         });
+        // We delay showing the popup so that fast Alt+Tab users aren't
+        // disturbed by the popup briefly flashing.
+        this._initialDelayTimeoutId = Mainloop.timeout_add(POPUP_DELAY_TIMEOUT,
+                                                           Lang.bind(this, function () {
+                                                               this.actor.opacity = 255;
+                                                               this._initialDelayTimeoutId = 0;
+                                                           }));
 
         return true;
     },
@@ -223,7 +235,7 @@ AltTabPopup.prototype = {
         let keysym = event.get_key_symbol();
         let event_state = Shell.get_event_state(event);
         let backwards = event_state & Clutter.ModifierType.SHIFT_MASK;
-        let action = global.screen.get_display().get_keybinding_action(event.get_key_code(), event_state);
+        let action = global.display.get_keybinding_action(event.get_key_code(), event_state);
 
         this._disableHover();
 
@@ -258,7 +270,7 @@ AltTabPopup.prototype = {
 
     _keyReleaseEvent : function(actor, event) {
         let [x, y, mods] = global.get_pointer();
-        let state = mods & Clutter.ModifierType.MOD1_MASK;
+        let state = mods & this._modifierMask;
 
         if (state == 0)
             this._finish();
@@ -395,6 +407,8 @@ AltTabPopup.prototype = {
             Mainloop.source_remove(this._motionTimeoutId);
         if (this._thumbnailTimeoutId != 0)
             Mainloop.source_remove(this._thumbnailTimeoutId);
+        if (this._initialDelayTimeoutId != 0)
+            Mainloop.source_remove(this._initialDelayTimeoutId);
     },
 
     /**
@@ -481,6 +495,10 @@ AltTabPopup.prototype = {
 
         this.actor.add_actor(this._thumbnails.actor);
 
+        // Need to force an allocation so we can figure out whether we
+        // need to scroll when selecting
+        this._thumbnails.actor.get_allocation_box();
+
         this._thumbnails.actor.opacity = 0;
         Tweener.addTween(this._thumbnails.actor,
                          { opacity: 255,
@@ -632,11 +650,10 @@ SwitcherList.prototype = {
                 this._items[this._highlighted].add_style_pseudo_class('selected');
         }
 
-        let monitor = Main.layoutManager.primaryMonitor;
-        let itemSize = this._items[index].allocation.x2 - this._items[index].allocation.x1;
-        let [posX, posY] = this._items[index].get_transformed_position();
-        posX += this.actor.x;
-        if (posX + itemSize > monitor.width + monitor.x)
+        let [absItemX, absItemY] = this._items[index].get_transformed_position();
+        let [result, posX, posY] = this.actor.transform_stage_point(absItemX, 0);
+        let [containerWidth, containerHeight] = this.actor.get_transformed_size();
+        if (posX + this._items[index].get_width() > containerWidth)
             this._scrollToRight();
         else if (posX < 0)
             this._scrollToLeft();
