@@ -284,7 +284,8 @@ FocusGrabber.prototype = {
         let source = event.get_source();
         switch (event.type()) {
             case Clutter.EventType.BUTTON_PRESS:
-                if (!this.actor.contains(source) && !Main.keyboard.actor.contains(source))
+                if (!this.actor.contains(source) &&
+                    !Main.layoutManager.keyboardBox.contains(source))
                     this.emit('button-pressed', source);
                 break;
             case Clutter.EventType.KEY_PRESS:
@@ -791,11 +792,12 @@ Notification.prototype = {
     },
 
     _bannerBoxAllocate: function(actor, box, flags) {
-        let [titleMinW, titleNatW] = this._titleLabel.get_preferred_width(-1);
-        let [titleMinH, titleNatH] = this._titleLabel.get_preferred_height(-1);
-        let [bannerMinW, bannerNatW] = this._bannerLabel.get_preferred_width(-1);
         let availWidth = box.x2 - box.x1;
 
+        let [titleMinW, titleNatW] = this._titleLabel.get_preferred_width(-1);
+        let [titleMinH, titleNatH] = this._titleLabel.get_preferred_height(availWidth);
+        let [bannerMinW, bannerNatW] = this._bannerLabel.get_preferred_width(availWidth);
+
         let titleBox = new Clutter.ActorBox();
         let titleBoxW = Math.min(titleNatW, availWidth);
         if (this._titleDirection == St.TextDirection.RTL) {
@@ -1494,8 +1496,11 @@ MessageTray.prototype = {
         // after notifications are done showing. However, we don't want that to happen for
         // transient sources, which are removed after the notification is shown, but are
         // not removed fast enough because of the callbacks to avoid the summary popping up.
-        // So we just don't add transient sources to this._newSummaryItems .
-        if (!source.isTransient)
+        // So we just don't add transient sources to this._newSummaryItems.
+        // We don't want that to happen for chat sources neither, because they
+        // can be added when the user starts a chat from Empathy and they are not transient.
+        // The notification will popup on incoming message anyway. See bug #657249.
+        if (!source.isTransient && !source.isChat)
             this._newSummaryItems.push(summaryItem);
 
         source.connect('notify', Lang.bind(this, this._onNotify));
