@@ -1291,6 +1291,9 @@ NMDeviceWireless.prototype = {
         apObj.accessPoints.splice(i, 1);
 
         if (apObj.accessPoints.length == 0) {
+            if (this._activeNetwork == apObj)
+                this._activeNetwork = null;
+
             if (apObj.item)
                 apObj.item.destroy();
 
@@ -1480,6 +1483,11 @@ NMDeviceWireless.prototype = {
     },
 
     _createNetworkItem: function(apObj, position) {
+        if(!apObj.accessPoints || apObj.accessPoints.length == 0) {
+            // this should not happen, but I have no idea why it happens
+            return;
+        }
+
         if(apObj.connections.length > 0) {
             if (apObj.connections.length == 1)
                 apObj.item = this._createAPItem(apObj.connections[0], apObj, false);
@@ -1742,13 +1750,6 @@ NMApplet.prototype = {
         if (wrapperClass) {
             let wrapper = new wrapperClass(this._client, device, this._connections);
 
-            wrapper._networkLostId = wrapper.connect('network-lost', Lang.bind(this, function(device) {
-                this._notifyForDevice(device, 'network-offline',
-                                      _("Connectivity lost"),
-                                      _("You are no longer connected to the network"),
-                                      // set critical urgency to popup the notification automatically
-                                      MessageTray.Urgency.CRITICAL);
-            }));
             wrapper._activationFailedId = wrapper.connect('activation-failed', Lang.bind(this, function(device, reason) {
                 // XXX: nm-applet has no special text depending on reason
                 // but I'm not sure of this generic message
@@ -1761,7 +1762,6 @@ NMApplet.prototype = {
                 this._syncSectionTitle(dev.category);
             }));
             wrapper._destroyId = wrapper.connect('destroy', function(wrapper) {
-                wrapper.disconnect(wrapper._networkLostId);
                 wrapper.disconnect(wrapper._activationFailedId);
                 wrapper.disconnect(wrapper._deviceStateChangedId);
                 wrapper.disconnect(wrapper._destroyId);
