@@ -11,17 +11,15 @@
 #include <gdk/gdkx.h>
 #include <glib.h>
 #include <gio/gio.h>
-#include <dbus/dbus-glib.h>
+#include <meta/display.h>
+#include <meta/group.h>
+#include <meta/window.h>
 
 #include "shell-app-usage.h"
 #include "shell-window-tracker.h"
 #include "shell-global.h"
 #include "shell-marshal.h"
 
-#include "display.h"
-#include "window.h"
-#include "group.h"
-
 /* This file includes modified code from
  * desktop-data-engine/engine-dbus/hippo-application-monitor.c
  * in the functions collecting application usage data.
@@ -83,7 +81,7 @@ struct _ShellAppUsage
   GObject parent;
 
   GFile *configfile;
-  DBusGProxy *session_proxy;
+  GDBusProxy *session_proxy;
   GdkDisplay *display;
   gulong last_idle;
   guint idle_focus_change_id;
@@ -107,7 +105,7 @@ G_DEFINE_TYPE (ShellAppUsage, shell_app_usage, G_TYPE_OBJECT);
 struct UsageData
 {
   /* Whether the application we're tracking is "transient", see
-   * shell_app_info_is_transient.
+   * shell_app_is_window_backed.
    */
   gboolean transient;
 
@@ -117,7 +115,7 @@ struct UsageData
 
 static void shell_app_usage_finalize (GObject *object);
 
-static void on_session_status_changed (DBusGProxy *proxy, guint status, ShellAppUsage *self);
+static void on_session_status_changed (GDBusProxy *proxy, guint status, ShellAppUsage *self);
 static void on_focus_app_changed (ShellWindowTracker *tracker, GParamSpec *spec, ShellAppUsage *self);
 static void ensure_queued_save (ShellAppUsage *self);
 static UsageData * get_app_usage_for_context_and_id (ShellAppUsage  *self,
@@ -309,15 +307,15 @@ increment_usage_for_app (ShellAppUsage *self,
 }
 
 static void
-on_app_state_changed (ShellWindowTracker *tracker,
-                      ShellApp           *app,
-                      gpointer            user_data)
+on_app_state_changed (ShellAppSystem *app_system,
+                      ShellApp       *app,
+                      gpointer        user_data)
 {
   ShellAppUsage *self = SHELL_APP_USAGE (user_data);
   UsageData *usage;
   gboolean running;
 
-  if (shell_app_is_transient (app))
+  if (shell_app_is_window_backed (app))
     return;
 
   usage = get_usage_for_app (self, app);
@@ -344,7 +342,7 @@ on_focus_app_changed (ShellWindowTracker *tracker,
 }
 
 static void
-on_session_status_changed (DBusGProxy      *proxy,
+on_session_status_changed (GDBusProxy      *proxy,
                            guint            status,
                            ShellAppUsage *self)
 {
@@ -375,13 +373,25 @@ on_session_status_changed (DBusGProxy      *proxy,
     }
 }
 
+static void
+session_proxy_signal (GDBusProxy *proxy, gchar *sender_name, gchar *signal_name, GVariant *parameters, gpointer user_data)
+{
+  if (g_str_equal (signal_name, "StatusChanged"))
+    {
+      guint status;
+      g_variant_get (parameters, "(u)", &status);
+      on_session_status_changed (proxy, status, SHELL_APP_USAGE (user_data));
+    }
+}
+
 static void
 shell_app_usage_init (ShellAppUsage *self)
 {
   ShellGlobal *global;
   char *shell_userdata_dir, *path;
-  DBusGConnection *session_bus;
+  GDBusConnection *session_bus;
   ShellWindowTracker *tracker;
+  ShellAppSystem *app_system;
 
   global = shell_global_get ();
 
@@ -389,16 +399,21 @@ shell_app_usage_init (ShellAppUsage *self)
 
   tracker = shell_window_tracker_get_default ();
   g_signal_connect (tracker, "notify::focus-app", G_CALLBACK (on_focus_app_changed), self);
-  g_signal_connect (tracker, "app-state-changed", G_CALLBACK (on_app_state_changed), self);
 
-  session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
-  self->session_proxy = dbus_g_proxy_new_for_name (session_bus, "org.gnome.SessionManager",
-                                                   "/org/gnome/SessionManager/Presence",
-                                                   "org.gnome.SessionManager");
-  dbus_g_proxy_add_signal (self->session_proxy, "StatusChanged",
-                           G_TYPE_UINT, G_TYPE_INVALID, G_TYPE_INVALID);
-  dbus_g_proxy_connect_signal (self->session_proxy, "StatusChanged",
-                               G_CALLBACK (on_session_status_changed), self, NULL);
+  app_system = shell_app_system_get_default ();
+  g_signal_connect (app_system, "app-state-changed", G_CALLBACK (on_app_state_changed), self);
+
+  session_bus = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, NULL);
+  self->session_proxy = g_dbus_proxy_new_sync (session_bus,
+                                               G_DBUS_PROXY_FLAGS_NONE,
+                                               NULL, /* interface info */
+                                               "org.gnome.SessionManager",
+                                               "/org/gnome/SessionManager/Presence",
+                                               "org.gnome.SessionManager",
+                                               NULL, /* cancellable */
+                                               NULL /* error */);
+  g_signal_connect (self->session_proxy, "g-signal", G_CALLBACK (session_proxy_signal), self);
+  g_object_unref (session_bus);
 
   self->last_idle = 0;
   self->currently_idle = FALSE;
@@ -434,6 +449,8 @@ shell_app_usage_finalize (GObject *object)
 
   g_object_unref (self->configfile);
 
+  g_object_unref (self->session_proxy);
+
   G_OBJECT_CLASS (shell_app_usage_parent_class)->finalize(object);
 }
 
@@ -495,7 +512,7 @@ shell_app_usage_get_most_used (ShellAppUsage   *self,
       const char *appid = iter->data;
       ShellApp *app;
 
-      app = shell_app_system_get_app (appsys, appid);
+      app = shell_app_system_lookup_app (appsys, appid);
       if (!app)
         continue;
 
@@ -656,7 +673,7 @@ idle_save_application_usage (gpointer data)
     {
       ShellApp *app;
 
-      app = shell_app_system_get_app (shell_app_system_get_default(), id);
+      app = shell_app_system_lookup_app (shell_app_system_get_default(), id);
 
       if (!app)
         continue;
