@@ -12,39 +12,56 @@
  * size, even if that would overflow the size allocated to the stack.
  */
 
+#include "config.h"
+
 #include "shell-stack.h"
 
 G_DEFINE_TYPE (ShellStack,
                shell_stack,
-               CLUTTER_TYPE_GROUP);
+               ST_TYPE_CONTAINER);
+
+static void
+shell_stack_paint (ClutterActor *actor)
+{
+  CLUTTER_ACTOR_CLASS (shell_stack_parent_class)->paint (actor);
+
+  clutter_container_foreach (CLUTTER_CONTAINER (actor),
+                             CLUTTER_CALLBACK (clutter_actor_paint),
+                             NULL);
+}
+
+static void
+shell_stack_pick (ClutterActor       *actor,
+                  const ClutterColor *pick)
+{
+  /* Chain up so we get a bounding box painted (if we are reactive) */
+  CLUTTER_ACTOR_CLASS (shell_stack_parent_class)->pick (actor, pick);
+
+  clutter_container_foreach (CLUTTER_CONTAINER (actor),
+                             CLUTTER_CALLBACK (clutter_actor_paint),
+                             NULL);
+}
 
 static void
 shell_stack_allocate (ClutterActor           *self,
                       const ClutterActorBox  *box,
                       ClutterAllocationFlags  flags)
 {
+  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
+  ClutterActorBox content_box;
   GList *children, *iter;
-  float width, height;
 
-  width = box->x2 - box->x1;
-  height = box->y2 - box->y1;
+  CLUTTER_ACTOR_CLASS (shell_stack_parent_class)->allocate (self, box, flags);
 
-  /* Chain up directly to ClutterActor to set actor->allocation.  We explicitly skip our parent class
-   * ClutterGroup here because we want to override the allocate function. */
-  (CLUTTER_ACTOR_CLASS (g_type_class_peek (clutter_actor_get_type ())))->allocate (self, box, flags);
+  st_theme_node_get_content_box (theme_node, box, &content_box);
 
-  children = clutter_container_get_children (CLUTTER_CONTAINER (self));
+  children = st_container_get_children_list (ST_CONTAINER (self));
   for (iter = children; iter; iter = iter->next)
     {
       ClutterActor *actor = CLUTTER_ACTOR (iter->data);
-      ClutterActorBox child_box;
-      child_box.x1 = 0;
-      child_box.x2 = width;
-      child_box.y1 = 0;
-      child_box.y2 = height;
+      ClutterActorBox child_box = content_box;
       clutter_actor_allocate (actor, &child_box, flags);
     }
-  g_list_free (children);
 }
 
 static void
@@ -54,12 +71,15 @@ shell_stack_get_preferred_height (ClutterActor *actor,
                                   gfloat *natural_height_p)
 {
   ShellStack *stack = SHELL_STACK (actor);
+  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
   gboolean first = TRUE;
   float min = 0, natural = 0;
   GList *children;
   GList *iter;
 
-  children = clutter_container_get_children (CLUTTER_CONTAINER (stack));
+  st_theme_node_adjust_for_width (theme_node, &for_width);
+
+  children = st_container_get_children_list (ST_CONTAINER (stack));
 
   for (iter = children; iter; iter = iter->next)
     {
@@ -93,7 +113,7 @@ shell_stack_get_preferred_height (ClutterActor *actor,
   if (natural_height_p)
     *natural_height_p = natural;
 
-  g_list_free (children);
+  st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
 }
 
 static void
@@ -103,12 +123,15 @@ shell_stack_get_preferred_width (ClutterActor *actor,
                                  gfloat *natural_width_p)
 {
   ShellStack *stack = SHELL_STACK (actor);
+  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (actor));
   gboolean first = TRUE;
   float min = 0, natural = 0;
   GList *iter;
   GList *children;
 
-  children = clutter_container_get_children (CLUTTER_CONTAINER (stack));
+  st_theme_node_adjust_for_height (theme_node, &for_height);
+
+  children = st_container_get_children_list (ST_CONTAINER (stack));
 
   for (iter = children; iter; iter = iter->next)
     {
@@ -141,17 +164,55 @@ shell_stack_get_preferred_width (ClutterActor *actor,
 
   if (natural_width_p)
     *natural_width_p = natural;
-  g_list_free (children);
+
+  st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
+}
+
+static gboolean
+shell_stack_navigate_focus (StWidget         *widget,
+                            ClutterActor     *from,
+                            GtkDirectionType  direction)
+{
+  ClutterActor *top_actor;
+  GList *children;
+
+  /* If the stack is itself focusable, then focus into or out of
+   * it, as appropriate.
+   */
+  if (st_widget_get_can_focus (widget))
+    {
+      if (from && clutter_actor_contains (CLUTTER_ACTOR (widget), from))
+        return FALSE;
+
+      clutter_actor_grab_key_focus (CLUTTER_ACTOR (widget));
+      return TRUE;
+    }
+
+  /* Otherwise, navigate into its top-most child only */
+  children = st_container_get_children_list (ST_CONTAINER (widget));
+  if (!children)
+    return FALSE;
+
+  top_actor = g_list_last (children)->data;
+  if (ST_IS_WIDGET (top_actor))
+    return st_widget_navigate_focus (ST_WIDGET (top_actor), from, direction, FALSE);
+  else
+    return FALSE;
 }
 
 static void
 shell_stack_class_init (ShellStackClass *klass)
 {
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
 
+  actor_class->paint = shell_stack_paint;
+  actor_class->pick = shell_stack_pick;
   actor_class->get_preferred_width = shell_stack_get_preferred_width;
   actor_class->get_preferred_height = shell_stack_get_preferred_height;
   actor_class->allocate = shell_stack_allocate;
+
+  widget_class->navigate_focus = shell_stack_navigate_focus;
 }
 
 static void
